package cd.casic.ci.common.pipeline.utils;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


/**
 * @Author：mianbin
 * @Package：cd.casic.ci.common.pipeline.utils
 * @Project：ops-pro
 * @name：EnvUtils
 * @Date：2025/03/27 15:20
 * @Filename：EnvUtils
 * @description：Todo
 */
public class EnvUtils {
    private static final Pattern tPattern = Pattern.compile("(\\$[{](?<single>[^$^{}]+)})|(\\$[{]{2}(?<double>[^$^{}]+)[}]{2})");

    public static String parseEnv(String command, Map<String, String> data) {
        return parseEnv(command, data, false, false);
    }

    public static String parseEnv(String command, Map<String, String> data, boolean replaceWithEmpty, boolean isEscape) {
        return parseEnv(command, data, replaceWithEmpty, isEscape, new HashMap<>());
    }

    public static String parseEnv(String command, Map<String, String> data, boolean replaceWithEmpty, boolean isEscape, Map<String, String> contextMap) {
        if (command == null || command.isEmpty()) {
            return command == null ? "" : command;
        }
        return parseTokenTwice(command, data, contextMap, replaceWithEmpty, isEscape, 1);
    }

    private static String parseTokenTwice(String command, Map<String, String> data, Map<String, String> contextMap, boolean replaceWithEmpty, boolean isEscape, int depth) {
        if (depth < 0) {
            return command;
        }
        Matcher matcher = tPattern.matcher(command);
        StringBuffer buff = new StringBuffer();
        while (matcher.find()) {
            String key = matcher.group("single");
            if (key == null) {
                key = matcher.group("double");
            }
            key = key.trim();
            String value = data.get(key);
            if (value == null && contextMap != null) {
                value = contextMap.get(key);
            }
            if (value == null) {
                value = !replaceWithEmpty ? matcher.group() : "";
            } else {
                if (depth > 0 && tPattern.matcher(value).find()) {
                    value = parseTokenTwice(value, data, contextMap, replaceWithEmpty, isEscape, depth - 1);
                } else if (isEscape) {
                    value = escapeSpecialWord(value);
                }
            }
            matcher.appendReplacement(buff, Matcher.quoteReplacement(value));
        }
        matcher.appendTail(buff);
        return buff.toString();
    }

    private static String escapeSpecialWord(String keyword) {
        String replaceWord = keyword;
        if (!keyword.isEmpty()) {
            String[] wordList = {"\\", "\""};
            for (String word : wordList) {
                if (replaceWord.contains(word)) {
                    replaceWord = replaceWord.replace(word, "\\" + word);
                }
            }
        }
        return replaceWord;
    }
}