package cd.casic.ci.process.api.process.pojo.trigger;

import cd.casic.ci.common.pipeline.pojo.IdValue;
import cd.casic.ci.log.scm.enums.ScmType;
import io.swagger.v3.oas.annotations.media.Schema;

import java.text.MessageFormat;
import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @Author：mianbin
 * @Package：cd.casic.ci.pipelineciprocessapi.process.pojo.trigger
 * @Project：ops-pro
 * @name：PipelineTriggerType
 * @Date：2025/03/24 17:27
 * @Filename：PipelineTriggerType
 * @description：Todo
 */
public enum PipelineTriggerType {
    @Schema(title = "SVN 代码库")
    CODE_SVN,
    @Schema(title = "GIT 代码库")
    CODE_GIT,
    @Schema(title = "Gitlab 代码库")
    CODE_GITLAB,
    @Schema(title = "Github 代码库")
    GITHUB,
    @Schema(title = "TGIT 代码库")
    CODE_TGIT,
    @Schema(title = "P4 代码库")
    CODE_P4,
    @Schema(title = "手动触发")
    MANUAL,
    @Schema(title = "定时触发")
    TIME_TRIGGER,
    @Schema(title = "服务触发")
    OPENAPI,
    @Schema(title = "流水线触发")
    PIPELINE,
    @Schema(title = "远程触发")
    REMOTE;

    // 通用触发类型
    private static final List<PipelineTriggerType> commonTriggerTypes = List.of(MANUAL, TIME_TRIGGER, REMOTE);

    public static List<IdValue> toMap(ScmType scmType, String userId) {
        List<PipelineTriggerType> triggerTypes;
        if (scmType == null) {
            triggerTypes = Arrays.asList(PipelineTriggerType.values());
        } else {
            triggerTypes = Arrays.stream(PipelineTriggerType.values())
                    .filter(it -> scmType.name().equals(it.name()))
                    .collect(Collectors.toList());
            triggerTypes.addAll(commonTriggerTypes);
        }
        return triggerTypes.stream()
                .map(it -> new IdValue(
                        it.name(),
                        MessageFormat.format(
                                "TRIGGER_TYPE_" + it.name(),
                                it.name(),
                                userId
                        )
                ))
                .collect(Collectors.toList());
    }

    public static ScmType toScmType(String triggerType) {
        return Arrays.stream(ScmType.values())
                .map(Enum::name)
                .anyMatch(triggerType::equals)
                ? ScmType.valueOf(triggerType)
                : null;
    }

    public static boolean webhookTrigger(String triggerType) {
        return List.of(
                CODE_SVN.name(),
                CODE_GIT.name(),
                CODE_GITLAB.name(),
                GITHUB.name(),
                CODE_TGIT.name(),
                CODE_P4.name()
        ).contains(triggerType);
    }
}
