package cd.casic.ci.log.scm.dal.pojo;


import cd.casic.ci.log.scm.dal.pojo.svn.*;
import cd.casic.framework.commons.util.spring.SpringUtils;
import cd.casic.ci.log.scm.config.GitConfig;
import cd.casic.ci.log.scm.config.SVNConfig;
import cd.casic.ci.log.scm.dal.pojo.git.GitApi;
import cd.casic.ci.log.scm.enums.CodeGitWebhookEvent;
import cd.casic.ci.log.scm.enums.CodeSvnRegion;
import cd.casic.ci.log.scm.enums.ScmType;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class ScmFactory {

    private static final GitApi gitApi = new GitApi();

    public static IScm getScm(String projectName, String url, ScmType type, String branchName, String privateKey, String passPhrase,
                              String token, CodeSvnRegion region, String userName, String event) {
        if (type == ScmType.CODE_SVN) {
            if (userName == null) {
                log.error("the svn username is null");
                //throw new ServiceException("the svn username is null");
            }
            if (privateKey == null) {
                log.error("The svn private key is null");
                //throw new ServiceException("The svn private key is null");
            }
            SVNConfig svnConfig = SpringUtils.getBean(SVNConfig.class);
            return new CodeSvnScmImpl(projectName, branchName, url, userName, privateKey, passPhrase, svnConfig);
        } else if (type == ScmType.CODE_GIT) {
            if (token == null) {
                log.error("The git token is null");
                //throw new ServiceException("The git token is null");
            }
            if (event != null && CodeGitWebhookEvent.find(event) == null) {
                log.error("The git event is invalid");
                //throw new ServiceException("The git event is invalid");
            }
            GitConfig gitConfig = SpringUtils.getBean(GitConfig.class);
            return new CodeGitScmImpl(projectName, branchName, url, privateKey, passPhrase, token, gitConfig, gitApi, event);
        } else if (type == ScmType.CODE_TGIT) {
            if (token == null) {
                log.error("The git token is null");
                //throw new ServiceException("The git token is null");
            }
            if (event != null && CodeGitWebhookEvent.find(event) == null) {
                log.error("The git event is invalid");
                //throw new ServiceException("The git event is invalid");
            }
            GitConfig gitConfig = SpringUtils.getBean(GitConfig.class);
            return new CodeTGitScmImpl(projectName, branchName, url, privateKey, passPhrase, token, gitConfig, gitApi, event);
        } else if (type == ScmType.CODE_GITLAB) {
            if (token == null) {
                log.error("The gitlab access token is null");
                //throw new ServiceException("The gitlab access token is null");
            }
            GitConfig gitConfig = SpringUtils.getBean(GitConfig.class);
            return new CodeGitlabScmImpl(projectName, branchName, url, privateKey, passPhrase, token, gitConfig, event);
        } else if (type == ScmType.CODE_P4) {
            if (passPhrase == null) {
                log.error("用户密码为空");
                //throw new ServiceException("用户密码为空");
            }
            if (userName == null) {
                log.error("用户名为空");
                //throw new ServiceException("用户名为空");
            }
            return new CodeP4ScmImpl(projectName, branchName, url, userName, passPhrase, event);
        } else {
            log.error("Unknown repo(" + type + ")");
            //throw new ServiceException("Unknown repo(" + type + ")");
        }
        return null;
    }
}
