package cd.casic.ci.log.scm.service.scm;


import cd.casic.ci.log.scm.config.GitConfig;
import cd.casic.ci.log.scm.config.P4Config;
import cd.casic.ci.log.scm.config.SVNConfig;
import cd.casic.ci.log.scm.dal.pojo.RevisionInfo;
import cd.casic.ci.log.scm.dal.pojo.ScmFactory;
import cd.casic.ci.log.scm.enums.CodeSvnRegion;
import cd.casic.ci.log.scm.enums.ScmType;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;


@Service
@Slf4j
public class ScmServiceImpl implements IScmService {

    @Resource
    private GitConfig gitConfig;

    @Resource
    private SVNConfig svnConfig;

    @Resource
    private P4Config p4Config;

    @Override
    public RevisionInfo getLatestRevision(String projectName, String url, ScmType type, String branchName, String privateKey, String paaPhrase, String token, CodeSvnRegion region, String userName) {
        log.info("[{}|{}|{}|{}] Start to get latest revision", projectName, url, type, userName);
        long startEpoch = System.currentTimeMillis();
        try {
            return ScmFactory.getScm(projectName, url, type, branchName, privateKey, paaPhrase, token, region, userName, null).getLatestRevision();
        } finally {
            log.info("It took {}ms to get the latest revision", System.currentTimeMillis() - startEpoch);
        }
    }

    @Override
    public void addWebhook(String projectName, String url, ScmType type, String privateKey, String passPhrase,
                           String token, CodeSvnRegion region, String userName, String event, String hookUrl) {
        log.info("[{}|{}|{}|{}|{}|{}|{}] Start to add web hook", projectName, url, type, region, userName, event, hookUrl);
        if (type == ScmType.CODE_SVN) {
            log.info("svn webhook api does not support, ignore");
            return;
        }
        long startEpoch = System.currentTimeMillis();
        try {
            String realHookUrl;
            if (StringUtils.isNoneBlank(hookUrl)) {
                realHookUrl = hookUrl;
            } else {
                if (type == ScmType.CODE_GIT) {
                    realHookUrl = gitConfig.gitHookUrl;
                } else if (type == ScmType.CODE_GITLAB) {
                    realHookUrl = gitConfig.gitlabHookUrl;
                } else if (type == ScmType.CODE_TGIT) {
                    realHookUrl = gitConfig.tGitHookUrl;
                } else if (type == ScmType.CODE_P4) {
                    realHookUrl = p4Config.p4HookUrl;
                } else {
                    throw new IllegalArgumentException("Unknown repository type (" + type + ") when add webhook");
                }
            }
            ScmFactory.getScm(projectName, url, type, null, privateKey, passPhrase, token, region, userName, event)
                    .addWebhook(realHookUrl);
        } finally {
            log.info("It took {}ms to add web hook", System.currentTimeMillis() - startEpoch);
        }
    }
}
