package cd.casic.ci.log.scm.service.tgit;


import cd.casic.framework.commons.util.http.OkhttpUtils;
import cd.casic.ci.log.scm.config.GitConfig;
import cd.casic.ci.log.scm.dal.pojo.git.GitUserInfo;
import cd.casic.ci.log.scm.dal.pojo.oauth.GitToken;
import cd.casic.ci.log.scm.enums.TokenTypeEnum;
import com.fasterxml.jackson.databind.ObjectMapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import okhttp3.*;
import org.springframework.stereotype.Service;

import java.io.IOException;

@Service
@Slf4j
public class TGitServiceImpl implements ITGitService {

    @Resource
    private GitConfig gitConfig;

    @Resource
    private ObjectMapper objectMapper;

    @Override
    public GitToken refreshToken(String userId, GitToken accessToken) {
        log.info("Start to refresh the token of user {}", userId);
        long startEpoch = System.currentTimeMillis();
        try {
            String url = gitConfig.tGitUrl + "/oauth/token" +
                    "?client_id=" + gitConfig.tGitClientId +
                    "&client_secret=" + gitConfig.tGitClientSecret +
                    "&grant_type=refresh_token" +
                    "&refresh_token=" + accessToken.getRefreshToken() +
                    "&redirect_uri=" + gitConfig.tGitWebhookUrl;
            Request request = new Request.Builder()
                    .url(url)
                    .post(RequestBody.create(MediaType.parse("application/x-www-form-urlencoded;charset=utf-8"), ""))
                    .build();
            Response response = OkhttpUtils.doHttp(request);
            okhttp3.ResponseBody body = response.body();
            assert body != null;
            String data = body.string();
            return objectMapper.readValue(data, GitToken.class);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            log.info("It took {}ms to refresh the token", System.currentTimeMillis() - startEpoch);
        }
        return null;
    }

    @Override
    public GitUserInfo getUserInfoByToken(String token, TokenTypeEnum tokenType) {
        log.info("Start to get the user info by token[{}]", token);
        long startEpoch = System.currentTimeMillis();
        try {
            StringBuilder url = new StringBuilder(gitConfig.tGitApiUrl + "/user");
            setToken(tokenType, url, token);
            log.info("getToken url>> {}", url);
            Request request = new Request.Builder()
                    .url(url.toString())
                    .get()
                    .build();
            Response response = OkhttpUtils.doHttp(request);
            ResponseBody body = response.body();
            assert body != null;
            String data = body.string();
            return objectMapper.readValue(data, GitUserInfo.class);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            log.info("It took {}ms to get the token", System.currentTimeMillis() - startEpoch);
        }
        return null;
    }

    private void setToken(TokenTypeEnum tokenType, StringBuilder url, String token) {
        if (TokenTypeEnum.OAUTH == tokenType) {
            url.append("?access_token=").append(token);
        } else {
            url.append("?private_token=").append(token);
        }
    }
}
