package cd.casic.pipeline.store.api.common;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.common.config.BusinessConfigRequest;
import cd.casic.pipeline.store.pojo.common.config.BusinessConfigResponse;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;


@Tag(name = "OP_STORE_BUSINESS_CONFIG", description = "OP-STORE-业务配置")
@RestController
@RequestMapping("/op/store/businessConfig")
public interface OpBusinessConfigResource {

    @Operation(summary = "添加业务配置（返回是否添加成功）")
    @PostMapping("/")
    CommonResult<Boolean> add(
            @Parameter(description = "业务配置信息请求报文体", required = true)
            BusinessConfigRequest businessConfigRequest
    );

    @Operation(summary = "更新业务配置信息（返回受影响的数据条数）")
    @PutMapping("/ids/{id}")
    CommonResult<Integer> update(
            @Parameter(description = "业务配置ID", required = true)
            @PathVariable("id")
            int id,
            @Parameter(description = "业务配置信息请求报文体", required = true)
            BusinessConfigRequest businessConfigRequest
    );

    @Operation(summary = "获取所有业务配置信息")
    @GetMapping("/list")
    CommonResult<List<BusinessConfigResponse>> listAllBusinessConfigs();

    @Operation(summary = "根据ID获取业务配置信息")
    @GetMapping("/{id}")
    CommonResult<BusinessConfigResponse> getBusinessConfigById(
            @Parameter(description = "业务配置ID", required = true)
            @PathVariable("id")
            int id
    );

    @Operation(summary = "根据ID删除业务配置信息（返回受影响的数据条数）")
    @DeleteMapping("/{id}")
    CommonResult<Integer> deleteBusinessConfigById(
            @Parameter(description = "业务配置ID", required = true)
            @PathVariable("id")
            int id
    );
}