package cd.casic.pipeline.store.api.container;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.pipeline.store.pojo.app.ContainerApp;
import cd.casic.pipeline.store.pojo.app.ContainerAppInfo;
import cd.casic.pipeline.store.pojo.app.ContainerAppRequest;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Tag(name = "OP_CONTAINER_APP", description = "OP-容器-编译环境")
@RestController
@RequestMapping("/op/pipeline/container/app")
public interface OpContainerAppResource {

    @Operation(summary = "添加编译环境")
    @PostMapping("/")
    CommonResult<Boolean> addContainerApp(
            @Parameter(description = "容器编译环境请求实体", required = true)
            ContainerAppRequest containerAppRequest
    );

    @Operation(summary = "删除编译环境")
    @DeleteMapping("/{id}")
    CommonResult<Boolean> deleteContainerAppById(
            @Parameter(description = "编译环境ID", required = true)
            @PathVariable("id")
            int id
    );

    @Operation(summary = "更新编译环境")
    @PutMapping("/{id}")
    CommonResult<Boolean> updateContainerApp(
            @Parameter(description = "编译环境ID", required = true)
            @PathVariable("id")
            int id,
            @Parameter(description = "容器编译环境请求实体", required = true)
            ContainerAppRequest containerAppRequest
    );

    @Operation(summary = "获取所有编译环境信息")
    @GetMapping("/")
    CommonResult<List<ContainerApp>> listContainerApps();

    @Operation(summary = "根据ID获取编译环境信息")
    @GetMapping("/{id}")
    CommonResult<ContainerAppInfo> getContainerAppById(
            @Parameter(description = "编译环境ID", required = true)
            @PathVariable("id")
            int id
    );
}