package cd.casic.pipeline.store.api.image;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.pipeline.store.pojo.common.comment.StoreCommentInfo;
import cd.casic.pipeline.store.pojo.common.comment.StoreCommentRequest;
import cd.casic.pipeline.store.pojo.common.comment.StoreCommentScoreInfo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;

@Tag(name = "USER_MARKET_IMAGE_COMMENT", description = "镜像-镜像评论")
@RestController
@RequestMapping("/user/market/image/comment")
public interface UserImageCommentResource {

    @Operation(summary = "获取镜像评论接口")
    @GetMapping("/comments/{commentId}")
    CommonResult<StoreCommentInfo> getStoreComment(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "评论ID", required = true)
            @PathVariable("commentId")
            String commentId
    );

    @Operation(summary = "获取镜像的评论列表")
    @GetMapping("/imageCodes/{imageCode}/comments")
    PageResult<StoreCommentInfo> getStoreComments(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "镜像代码", required = true)
            @PathVariable("imageCode")
            String imageCode,
            @Parameter(description = "页码", required = true)
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页数量", required = true)
            @RequestParam("pageSize")
            Integer pageSize
    );

    @Operation(summary = "获取镜像的评分详情")
    @GetMapping("/score/imageCodes/{imageCode}")
    CommonResult<StoreCommentScoreInfo> getImageCommentScoreInfo(
            @Parameter(description = "镜像代码", required = true)
            @PathVariable("imageCode")
            String imageCode
    );

    @Operation(summary = "新增镜像评论")
    @PostMapping("/imageIds/{imageId}/imageCodes/{imageCode}/comment")
    CommonResult<StoreCommentInfo> addImageComment(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "镜像ID", required = true)
            @PathVariable("imageId")
            String imageId,
            @Parameter(description = "镜像代码", required = true)
            @PathVariable("imageCode")
            String imageCode,
            @Parameter(description = "评论信息请求报文体", required = true)
            StoreCommentRequest storeCommentRequest
    );

    @Operation(summary = "更新镜像评论")
    @PutMapping("/comments/{commentId}")
    CommonResult<Boolean> updateStoreComment(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "评论ID", required = true)
            @PathVariable("commentId")
            String commentId,
            @Parameter(description = "评论信息请求报文体", required = true)
            StoreCommentRequest storeCommentRequest
    );

    @Operation(summary = "评论点赞/取消点赞")
    @PutMapping("/praise/{commentId}")
    CommonResult<Integer> updateStoreCommentPraiseCount(
            @Parameter(description = "userId", required = true)
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "评论ID", required = true)
            @PathVariable("commentId")
            String commentId
    );
}