package cd.casic.ci.ticket.api;

import cd.casic.framework.commons.pojo.CommonResult;
import cd.casic.framework.commons.pojo.PageResult;
import cd.casic.ci.ticket.dal.pojo.CredentialWithPermission;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.web.bind.annotation.*;

import static cd.casic.ci.common.pipeline.constant.OpsHeader.AUTH_HEADER_USER_ID;

/**
 * @Author：mianbin
 * @Package：cd.casic.pipeline.api
 * @Project：ops-pro
 * @name：OpCredentialResource
 * @Date：2025/03/22 16:05
 * @Filename：OpCredentialResource
 */
@Tag(name = "OP_CREDENTIAL", description = "OP-凭据资源")
@RestController
@RequestMapping("/op/credentials")
public interface OpCredentialApi {
    @Operation(summary = "获取凭据列表")
    @GetMapping("/projects/{projectId}/")
    CommonResult<PageResult<CredentialWithPermission>> list(
            @Parameter(description = "用户ID", required = true, example = "admin")
            @RequestHeader(AUTH_HEADER_USER_ID)
            String userId,
            @Parameter(description = "项目ID", required = true)
            @PathVariable("projectId")
            String projectId,
            @Parameter(description = "凭证类型列表，用逗号分隔", required = false, example = "")
            @RequestParam("credentialTypes")
            String credentialTypesString,
            @Parameter(description = "第几页", required = false, example = "1")
            @RequestParam("page")
            Integer page,
            @Parameter(description = "每页多少条", required = false, example = "20")
            @RequestParam("pageSize")
            Integer pageSize,
            @Parameter(description = "关键字", required = false)
            @RequestParam("keyword")
            String keyword
    );
}
